@@ -11,15 +11,6 @@
 honestRF <- function(...) forestry(...)
 
 
-#' @title Honest Random Forest
-#' @description This function is deprecated and only exists for backwards
-#'   backwards compatibility. The function you want to use is `autoforestry`.
-#' @param ... parameters which are passed directly to `autoforestry`
-#' @return A `forestry` object
-#' @export autohonestRF
-autohonestRF <- function(...) autoforestry(...)
-
-
 # -- Random Forest Constructor -------------------------------------------------
 #' @title forestry class
 #' @name forestry-class
